import { Component, OnInit, OnDestroy } from '@angular/core';
import { SearchService } from './shared-services/search.service';
import { FilterListsService } from './shared-services/filter-lists.service';
import { SettingsService } from './shared-services/settings.service';
import { CatchAllListsService } from './shared-services/catch-all-lists.service';
import { Subscription } from 'rxjs/Subscription';
import { Router } from '@angular/router';
import { environment } from './../environments/environment';

@Component({
  selector: 'app-root',
  templateUrl: './app.component.html',
  styleUrls: ['./app.component.css']
})
export class AppComponent implements OnInit, OnDestroy {
  private initializeCompleteSubscription: Subscription;
  private environmentChangedSubscription: Subscription;

  constructor(private router: Router, private searchService: SearchService,
    private filterListsService: FilterListsService, private settingsService: SettingsService,
    private catchAllListsService: CatchAllListsService) {
    this.initializeCompleteSubscription = this.catchAllListsService.getInitializeCompleteSubscription()
      .subscribe(message => {
        this.settingsService.setSelectedDatabaseEnv(environment.defaultDatabaseEnv);
        this.settingsService.setSelectedSharepointEnv(environment.defaultSharepointEnv);
      });

    // If the Database or Sharepoint environment is changed, reset the app.
    this.environmentChangedSubscription = this.settingsService.getEnvironmentChangedSubscription()
      .subscribe(message => {
        this.filterListsService.reset();
        this.settingsService.filterSettings.reset();
        this.settingsService.isBasic = true;
        this.searchService.reset();
        this.router.navigateByUrl('/home');
      });
  }

  ngOnInit() {
    this.catchAllListsService.initialize();
  }

  ngOnDestroy() {
    // Prevent memory leaks.
    this.initializeCompleteSubscription.unsubscribe();
    this.environmentChangedSubscription.unsubscribe();
  }
}
