import { Component, OnInit, OnDestroy } from '@angular/core';
import { DialogService } from '../shared-services/dialog.service';
import { Observable } from 'rxjs/Observable';
import { Subject } from 'rxjs/Subject';
import { Subscription } from 'rxjs/Subscription';

declare var jQuery: any;

@Component({
  selector: 'app-dialog',
  templateUrl: './dialog.component.html',
  styleUrls: ['./dialog.component.css']
})
export class DialogComponent implements OnInit, OnDestroy {
  private tooManyMatchesSubscription: Subscription;
  private missingSearchFilterSubscription: Subscription;
  private communicationErrorSubscription: Subscription;

  constructor(private dialogService: DialogService) {
    this.tooManyMatchesSubscription = this.dialogService.getTooManyMatchesSubscription().subscribe(message => {
      // Set the dialog's header and body.
      jQuery('#dlgHeader').html('Too Many Matches');
      jQuery('#dlgBody').html('This site can display up to 2500 matches, but ' + message +
        ' matches were found.<br/><br/>Please specify additional search criteria.');

      // Show the dialog.
      jQuery('#modalDlg').modal('show');
    });

    this.missingSearchFilterSubscription = this.dialogService.getMissingSearchFilterSubscription().subscribe(message => {
      // Set the dialog's header and body.
      jQuery('#dlgHeader').html('Missing Search Filter');
      jQuery('#dlgBody').html('You must specify at least one search filter.');

      // Show the dialog.
      jQuery('#modalDlg').modal('show');
    });

    this.communicationErrorSubscription = this.dialogService.getCommunicationErrorSubscription().subscribe(message => {
      // Set the dialog's header and body.
      jQuery('#dlgHeader').html('Web Server Communication Error');
      jQuery('#dlgBody').html('Unable to communicate with the web server.');

      // Show the dialog.
      jQuery('#modalDlg').modal('show');
    });
  }

  ngOnInit() {
    jQuery('#modalDlg').focus();
  }

  ngOnDestroy() {
		// Prevent memory leaks.
    this.tooManyMatchesSubscription.unsubscribe();
    this.missingSearchFilterSubscription.unsubscribe();
    this.communicationErrorSubscription.unsubscribe();
  }
}
