import { Component, OnInit } from '@angular/core';
import { SearchService } from '../shared-services/search.service';
import { FilterListsService } from '../shared-services/filter-lists.service';
import { SettingsService } from '../shared-services/settings.service';
import { CatchAllListsService } from '../shared-services/catch-all-lists.service';
import { CMF } from '../shared-types/cmf';
import { FILTERSETTINGS } from '../shared-types/filter-settings';

declare var jQuery: any;

@Component({
  selector: 'app-filters',
  templateUrl: './filters.component.html',
  styleUrls: ['./filters.component.css']
})

export class FiltersComponent implements OnInit {
  private comboOptions: IgCombo;
  private salesOrgComboOptions: IgCombo;
  private cmfPlaceholder = this.getPlaceholderText();

  constructor(private searchService: SearchService, private filterListsService: FilterListsService,
    private settingsService: SettingsService, private catchAllListsService: CatchAllListsService) {

    this.comboOptions = {
      mode: 'editable',
      textKey: 'Text',
      valueKey: 'Text',
      allowCustomValue: false,
      autoSelectFirstMatch: false,
      placeHolder: 'select...',
      enableClearButton: false,
      autoComplete: false,
      filteringType: 'local',
      filteringCondition: 'contains',
      highlightMatchesMode: 'contains',
      width: '100%'
    };

    this.salesOrgComboOptions = {
      mode: 'editable',
      textKey: 'Text',
      valueKey: 'Text',
      allowCustomValue: false,
      autoSelectFirstMatch: false,
      placeHolder: 'select...',
      enableClearButton: false,
      autoComplete: false,
      filteringType: 'local',
      filteringCondition: 'contains',
      highlightMatchesMode: 'contains',
      width: '100%',
      multiSelection: {
          enabled: true,
          showCheckboxes: true
      },
      selectionChanged: function (evt, ui) {
        // TODO::TJE Hack alert, I can't figure out how to get the angular binding
        // to work with a multi-select igcombo, so this is the workaround that I've
        // come up with.
        settingsService.filterSettings.setSalesOrg(ui.items);
      }
    };
  }

  ngOnInit() {
    // TODO::TJE Since binding is not setup properly, I need to select items
    // in SalesOrgCombo based on this.settingsService.filterSettings.salesOrgsStr,
    // but I can't get it to work.
    // initialSelectedItems:  [ { value: 'AM07' } ],
    //jQuery('#SalesOrgCombo').igCombo('option', 'selectedItems', []);
  }

  private getPlaceholderText() {
    if (this.settingsService.filterSettings.IsExactMatch === true) {
      return 'Enter a CMF Id';
    } else {
      return 'Enter a whole or partial CMF Id';
    }
  }

  private onExactMatchClicked() {
    this.settingsService.filterSettings.IsExactMatch = !this.settingsService.filterSettings.IsExactMatch;
    this.cmfPlaceholder = this.getPlaceholderText();
  }

  private onSearchClick() {
    this.searchService.doSearch(this.settingsService.filterSettings, this.settingsService.isBasic);
  }

  private onClearClick() {
    this.settingsService.filterSettings.reset();

    // TODO::TJE Since binding is not setup properly, I need to need to manually
    // clear the SalesAreaCombo selections, but I can't get it to work.
    //jQuery('#SalesOrgCombo').igCombo('option', 'selectedItems', []);
  }

  private onBasicTabSelected() {
    this.settingsService.isBasic = true;
  }

  private onAdvancedTabSelected() {
    this.settingsService.isBasic = false;
    this.filterListsService.initialize();
  }

  private isSearchValid(): boolean {
    if (this.settingsService.isBasic) {
      return !this.isNullOrWhitespace(this.settingsService.filterSettings.CmfIdValue, true);
    } else {
      return !(this.isNullOrWhitespace(this.settingsService.filterSettings.BusinessPurpose, false) &&
        this.isNullOrWhitespace(this.settingsService.filterSettings.City, true) &&
        this.isNullOrWhitespace(this.settingsService.filterSettings.CmfIdValue, true) &&
        this.isNullOrWhitespace(this.settingsService.filterSettings.Country, false) &&
        this.isNullOrWhitespace(this.settingsService.filterSettings.CustomerGroupCode1, false) &&
        this.isNullOrWhitespace(this.settingsService.filterSettings.CustomerGroupCode2, false) &&
        this.isNullOrWhitespace(this.settingsService.filterSettings.CustomerGroupCode3, false) &&
        this.isNullOrWhitespace(this.settingsService.filterSettings.CustomerGroupCode4, false) &&
        this.isNullOrWhitespace(this.settingsService.filterSettings.CustomerGroupCode5, false) &&
        this.isNullOrWhitespace(this.settingsService.filterSettings.CustomerName, true) &&
        this.isNullOrWhitespace(this.settingsService.filterSettings.DistributionChannel, false) &&
        this.isNullOrWhitespace(this.settingsService.filterSettings.Incoterm1, false) &&
        this.isNullOrWhitespace(this.settingsService.filterSettings.Incoterm2, true) &&
        this.isNullOrWhitespace(this.settingsService.filterSettings.IndustryKeyCode, false) &&
        this.isNullOrWhitespace(this.settingsService.filterSettings.PartnerFunction, false) &&
        this.isNullOrWhitespace(this.settingsService.filterSettings.PlantCode, false) &&
        this.isNullOrWhitespace(this.settingsService.filterSettings.SalesGroup, false) &&
        this.isNullOrWhitespace(this.settingsService.filterSettings.SalesOffice, false) &&
        this.isNullOrWhitespace(this.settingsService.filterSettings.SalesOrgs, false) &&
        this.isNullOrWhitespace(this.settingsService.filterSettings.SearchTerm, true) &&
        this.isNullOrWhitespace(this.settingsService.filterSettings.State, false) &&
        this.isNullOrWhitespace(this.settingsService.filterSettings.StreetAddress, true) &&
        this.isNullOrWhitespace(this.settingsService.filterSettings.Attribute8, false));
    }
  }

  private isClearValid(): boolean {
    if (this.settingsService.isBasic) {
      return !this.isNullOrWhitespace(this.settingsService.filterSettings.CmfIdValue, true);
    } else {
      return !(this.isNullOrWhitespace(this.settingsService.filterSettings.BusinessPurpose, false) &&
        this.isNullOrWhitespace(this.settingsService.filterSettings.City, true) &&
        this.isNullOrWhitespace(this.settingsService.filterSettings.CmfIdValue, true) &&
        this.isNullOrWhitespace(this.settingsService.filterSettings.Country, false) &&
        this.isNullOrWhitespace(this.settingsService.filterSettings.CustomerGroupCode1, false) &&
        this.isNullOrWhitespace(this.settingsService.filterSettings.CustomerGroupCode2, false) &&
        this.isNullOrWhitespace(this.settingsService.filterSettings.CustomerGroupCode3, false) &&
        this.isNullOrWhitespace(this.settingsService.filterSettings.CustomerGroupCode4, false) &&
        this.isNullOrWhitespace(this.settingsService.filterSettings.CustomerGroupCode5, false) &&
        this.isNullOrWhitespace(this.settingsService.filterSettings.CustomerName, true) &&
        this.isNullOrWhitespace(this.settingsService.filterSettings.DistributionChannel, false) &&
        this.isNullOrWhitespace(this.settingsService.filterSettings.Incoterm1, false) &&
        this.isNullOrWhitespace(this.settingsService.filterSettings.Incoterm2, true) &&
        this.isNullOrWhitespace(this.settingsService.filterSettings.IndustryKeyCode, false) &&
        this.isNullOrWhitespace(this.settingsService.filterSettings.PartnerFunction, false) &&
        this.isNullOrWhitespace(this.settingsService.filterSettings.PlantCode, false) &&
        this.isNullOrWhitespace(this.settingsService.filterSettings.SalesGroup, false) &&
        this.isNullOrWhitespace(this.settingsService.filterSettings.SalesOffice, false) &&
        this.isNullOrWhitespace(this.settingsService.filterSettings.SalesOrgs, false) &&
        this.isNullOrWhitespace(this.settingsService.filterSettings.SearchTerm, true) &&
        this.isNullOrWhitespace(this.settingsService.filterSettings.State, false) &&
        this.isNullOrWhitespace(this.settingsService.filterSettings.StreetAddress, true) &&
        this.isNullOrWhitespace(this.settingsService.filterSettings.Attribute8, false));
    }
  }

  private isNullOrWhitespace(input: string, checkWhitespace: boolean) {
    if (checkWhitespace) {
      return !input || !input.trim();
    } else {
      return input === null;
    }
  }
}
