import { Injectable, Output, EventEmitter } from '@angular/core';
import { Headers, Http, Response, RequestOptions, URLSearchParams } from '@angular/http';
import 'rxjs/add/operator/toPromise';
import 'rxjs/Rx';
import { Observable } from 'rxjs/Observable';
import { Subject } from 'rxjs/Subject';
import { CMF } from '../shared-types/cmf';
import { CHANGETYPEMAPPING } from '../shared-types/change-type-mapping';
import { OWLENVIRONMENT } from '../shared-types/owlenvironment';
import { FILTERSETTINGS } from '../shared-types/filter-settings';
import { BusyIndicatorService } from '../shared-services/busy-indicator.service';
import { DialogService } from '../shared-services/dialog.service';
import { CustomQueryEncoderHelper } from '../shared-helpers/customQueryEncoderHelper';
import { environment } from '../../environments/environment';
import { shim } from 'promise.prototype.finally';

@Injectable()
export class CatchAllListsService {
  private initializeCompleteSubject = new Subject<any>();

  sharepointEnvList: OWLENVIRONMENT[];
  databaseEnvList: OWLENVIRONMENT[];
  supportedPartnerFuncList: string[];
  changeTypeMappingList: CHANGETYPEMAPPING[];

  constructor(private http: Http, private busyIndicatorService: BusyIndicatorService, private dialogService: DialogService) {
    shim();
  }

  initialize() {
    const params: URLSearchParams = new URLSearchParams('', new CustomQueryEncoderHelper());

    this.busyIndicatorService.showBusyIndicator('Intializing, please wait...');

    const headers = new Headers({
      'Accept': 'application/json'
    });

    // Set the request options.
    const requestOptions = new RequestOptions();
    requestOptions.headers = headers;
    requestOptions.search = params;
    requestOptions.withCredentials = true;

    let promise: Promise<any>;

    promise = this.http.get(environment.catchAllListsWebApiUrl, requestOptions)
      .toPromise()
      .then(response => response.json())
      .catch((err) => {
        this.handleError(err, this.dialogService);
      })
      .finally(() => {
        this.busyIndicatorService.hideBusyIndicator();
      });

    promise.then(results => {
      if (results) {
        this.sharepointEnvList = results.SharepointList;
        this.databaseEnvList = results.DatabaseList;
        this.supportedPartnerFuncList = results.SupportedPartnerFuncList;
        this.changeTypeMappingList = results.ChangeTypeMappingList;

        this.initializeCompleteSubject.next(true);
      }
    });
  }

  private handleError(error: any, dialogService: DialogService): Promise<any> {
    console.log('Error Encountered In CatchAllListsService');
    console.error(error);

    dialogService.showCommunicationErrorDialog();
    return Promise.reject(error.message || error);
  }

  getInitializeCompleteSubscription(): Observable<any> {
    return this.initializeCompleteSubject.asObservable();
  }
}
