import { Injectable } from '@angular/core';
import { Headers, Http, Response, RequestOptions, URLSearchParams } from '@angular/http';
import 'rxjs/add/operator/toPromise';
import 'rxjs/Rx';
import { FILTERLISTVALUES } from '../shared-types/filter-list-values';
import { BusyIndicatorService } from '../shared-services/busy-indicator.service';
import { CatchAllListsService } from '../shared-services/catch-all-lists.service';
import { SettingsService } from '../shared-services/settings.service';
import { DialogService } from '../shared-services/dialog.service';
import { environment } from '../../environments/environment';

@Injectable()
export class FilterListsService {
  private isLoading = false;
  filterListValues: FILTERLISTVALUES = new FILTERLISTVALUES();

  constructor(private http: Http, private busyIndicatorService: BusyIndicatorService, private dialogService: DialogService,
    private settingsService: SettingsService, private catchAllListsService: CatchAllListsService) { }

  initialize() {
    const params: URLSearchParams = this.getURL();

    if (!this.isLoading && (!this.filterListValues || !this.filterListValues.PartnerFunctionList ||
      this.filterListValues.PartnerFunctionList.length === 0)) {
      this.busyIndicatorService.showBusyIndicator('Retrieving filter list values, please wait...');
      this.isLoading = true;
      let promise: Promise<FILTERLISTVALUES>;

      const headers = new Headers({
        'Accept': 'application/json',
        'Content-Type': 'application/x-www-form-urlencoded'
      });

      // Set the request options.
      const requestOptions = new RequestOptions();
      requestOptions.headers = headers;
      requestOptions.search = params;
      requestOptions.withCredentials = true;

      promise = this.http.get(environment.filterListsWebApiUrl, requestOptions)
        .toPromise()
        .then(response => response.json() as FILTERLISTVALUES)
        .catch((err) => {
          this.handleError(err, this.dialogService);
        })
        .finally(() => {
          this.busyIndicatorService.hideBusyIndicator();
          this.isLoading = false;
        });

      promise.then(results => {
        if (results) {
          this.filterListValues = results;
        }
      });
    }
  }

  reset() {
    this.isLoading = false;
    this.filterListValues = new FILTERLISTVALUES();
  }

  private handleError(error: any, dialogService: DialogService): Promise<any> {
    console.log('Error Encountered In FilterListsService');
    console.error(error);

    dialogService.showCommunicationErrorDialog();
    return Promise.reject(error.message || error);
  }

  private getURL(): URLSearchParams {
    const params: URLSearchParams = new URLSearchParams();

    params.set('DatabaseEnv', this.settingsService.selectedDatabaseEnv.Id.toString());
    params.set('SharepointEnv', this.settingsService.selectedSharepointEnv.Id.toString());

    return params;
  }
}
